import os
import pickle

SESSION_DIR = ''
SESSION_POSTFIX = 'session'
SESSION_PREFIX = ''


class SessionDoesNotExistsException(Exception):
    pass


class SessionExistsException(Exception):
    pass


class InstanceIsNotProvided(Exception):
    pass


class SessionBuilder:

    def __init__(self, session_name, instance=None):
        self.session_name = session_name
        self._session = None
        self._session_file_dir = None
        self._instance = instance

    @property
    def session_file_dir(self):
        if self._session_file_dir is not None:
            return self._session_file_dir
        return os.path.join(SESSION_DIR, '{}{}.{}'.format(SESSION_PREFIX, self.session_name, SESSION_POSTFIX))

    def _read_session_file(self):
        try:
            with open(self.session_file_dir, 'rb') as session_file:
                self._session = pickle.load(session_file)
            return self._session
        except FileNotFoundError:
            raise SessionDoesNotExistsException

    def _create_session_file(self):
        with open(self.session_file_dir, 'wb+') as session_file:
            pickle.dump(self._instance, session_file)

    def create(self):
        # this will avoid overriding session which already exists
        if self.exists():
            raise SessionExistsException
        if self._instance is None:
            raise InstanceIsNotProvided
        # creates a new session file and puts session instance in it
        self._create_session_file()

    def update(self, instance):
        self._instance = instance
        self._create_session_file()

    def get(self, force_update=False):
        # TODO: CHECK FOR IF FILE IS A VALID FILE
        if self._session is not None and not force_update:
            return self._session
        else:
            return self._read_session_file()

    def delete(self):
        if not self.exists():
            raise SessionDoesNotExistsException
        if os.path.isfile(self.session_file_dir):
            os.unlink(self.session_file_dir)

    def exists(self):
        return os.path.exists(self.session_file_dir)


def delete_all_sessions():
    import os
    for the_file in os.listdir(SESSION_DIR):
        file_path = os.path.join(SESSION_DIR, the_file)
        if os.path.isfile(file_path):
            os.unlink(file_path)
